<?php
namespace api;

include_once("../lib/f_proxy.php");

p_include("core/php/crud.php","");
p_include("core/php/util.php","");

use core;

class Link
{
    public function __construct($parent_type) 
    {
        $settings = core\getSettings("../lib"); 
        $db = core\getVal($settings,"db");

        $this->crud = new  core\Crud($db);       
        
        $this->parent_type = $parent_type;
        $this->table = "link_".$parent_type;
        
        $this->build();
    }

    function  build()
	{
		$fields=
        [
            "p_id VARCHAR(255)",
            "c_id VARCHAR(255)",
            "c_type VARCHAR(255)",
            "data JSON"
        ];


		$res = $this->crud-> buildTable($this->table,$fields);


        $res = json_encode($res);
    }

    function prepareData($input)
    {
        if(!$input) return $input;
        $data = core\getVal($input,"data",[]);
        $cols = ["id","p_id","c_id","c_type"];

        return core\filterArray($data,$cols);
    }

    public function save($input=[])
    {
        $res = [];
    
        $id = core\getVal($input,'id',null);
        $data = core\getVal($input,'data',[]);
        
        $req_data = $data;
    
        if ($id) 
        {
            $req_data["id"] = $id;
        }
        $req_data["p_id"] = core\getVal($input, "parent_id");
        $req_data["c_id"] = core\getVal($input, "child_id");
        $req_data["c_type"] = core\getVal($input, "child_type");
        
        // Check if a link with the specified criteria already exists
        $link_res = $this->getLink($req_data);
        $id = core\getVal($link_res, "link_id");

        if ($id) 
        {
            // Update the existing link
            $req_data["id"] = $id;
        }

        $req_data = $this->prepareData(["data"=>$req_data]);
        
        $req = 
        [
            "table" => $this->table,
            "data" => $req_data,
        ];

        $res = $this->crud->save($req);
        
        return $res;
    }
 
    public function list($input = [])
    {
        $parent = core\getVal($input,"parent");
        $child = core\getVal($input,"child");
        $child_types = core\getVal($input,'child_types',[]);

        $table = $this->table;
        $where = core\getVal($input,"where","");
        $fields = [];
      
        $join = "";

        if($parent)
        {
            $p_id = core\getVal($parent,"id");
        
            $where.=" WHERE p_id = '$p_id' " ;
        }
        else return [];

        if(count($child_types)==0)
        {
            $c_type = core\getVal($child,"type");
            $child_types[]=$c_type;
        }
             
        $fields[] ="BIN_TO_UUID($table.id) as link_id";
        $fields[] =" $table.data as link";
        $fields[] =" c_type as type";

        $order_fields = [];
        $group = [];
        $group[] = "$table.id";
        foreach ($child_types as $k => $v) 
        {
            $join .= " Inner JOIN $v on $v.id = UUID_TO_BIN($table.c_id) ";
            $fields[] ="$v.data as data";
            $fields[] ="BIN_TO_UUID($v.id) as id";
            $group[] = "$v.id";

            $order_fields[] = " $v.data->'$.name' ";
        }
        
        $types = implode("','", $child_types);
        $where.=" WHERE c_type in ('$types')" ;
        
        $input["where"] = $where;
        $input["table"] = $table;
        $input["fields"] = $fields;
        $input["order_fields"] =  $order_fields;
        $input["join"] = $join;
        // $input["group"] = " Group By link.id,entity.id";
        $input["group"] =  " Group By ".implode(",", $group)." ";

        $res = $this->crud->read($input);
        // print_r($res);
        return $res;
    }

    public function delete($input=[])
    {
        $req["data"] = $input;
        $req["table"] = $this->table;
        $res = $this->crud->delete($req);
        return  $res;
    }

    public function getCount()
    {
        $req["table"] = $this->table;
        $req["type"] = $this->type;
        

        return $this->crud->getCount($req);
    }

    public function getLink($input = [])
    {
        $table = core\getVal($input, "table",$this->table); 
       
        $p_id = core\getVal($input, "p_id");
        $c_id = core\getVal($input, "c_id");
        $c_type = core\getVal($input, "c_type");

        $where = "";

        if($p_id) $where_p_id = " p_id = '$p_id'"; 
        if($c_id) $where_c_id = " c_id = '$c_id'";
        if($c_type) $where_c_type = " c_type = '$c_type'";

        if($where_p_id) $where.= $where_p_id;

        if($where_c_id)
        {
            if($where) $where.= " AND ".$where_c_id;
            else $where.= $where_c_id;
        }

        if($where_c_type)
        {
            if($where) $where.= " AND ".$where_c_type;
            else $where.= $where_c_type;
        }

        if(!$where) return [];

        $input["where"] = "WHERE ". $where;

        $fields = [];
        $fields[] = "IFNULL(BIN_TO_UUID($table.id), '') as link_id";
        $fields[] = "$table.p_id as p_id";
        $fields[] = "$table.c_id as c_id";
        $fields[] = "$table.c_type as c_type";
        $fields[] = "'$table' as type";

        // $input["where"] = $where;
        $input["table"] = $table;
        $input["fields"] = $fields;

        $res = $this->crud->read($input);
        // print_r($res);

        $res = core\getval( $res,"data",[]);
        return reset($res);
     
    }
}